import { ECU } from "syrus4-nodejs"
import * as TPMS from "./tpms.js"
// lengths of the TAIP variables to filled with commas when no middle data received
export const ECU_LENGTHS = {
	ZA: 9,
	ZE: 9,
	ZS: 11,
	ZH: 5,
	ZX: 7,
	ZY: 7,
	BTT: 16,
	BTV: 110,
	BTP: 48,

};

export const ECU_LIST = ECU.getECUList();

// Helper function to contruct all the extended tag associated with ECU monitor
const COMBINED_REGEX = /^[A-Z]+(\.|-)\d+$/
export function ECUToTAIP(data) {
	let txt = "";
	const combined = {}
	const complex = {}
	const ecuFlag = data["$io.ign"] ? "T" : "F"
	const partials = []
	for (const key in data) {
		if (!key.startsWith("$ecu.")) continue

		let value = data[key];
		const element = ECU_LIST[key.replace("$ecu.", "")]

		if (!(element && element.$taip_tag)) continue
		if (!element.$type) element.$type = 'number'
		if (!element.$taip_multiplier) element.$taip_multiplier = 1
		if (element.$type == 'number') {
			let typed = Number(value) * element.$taip_multiplier
			if (!element.$taip_float) typed = parseInt(typed)
			if (!isNaN(typed)) value = typed
		}
		if (!element.$name) element.$name = element.$taip_tag

		let { $taip_tag } = element
		if (!element.$tokenizer && COMBINED_REGEX.test($taip_tag)) {
			let sep = $taip_tag.includes(".") ? '.' : '-'
			const [tag, pos] = $taip_tag.split(sep)
			if (sep == '-') partials.push(tag)
			if (!combined[tag]) {
				combined[tag] = []
				if (ECU_LENGTHS[tag] > combined[tag].length)
					combined[tag].length = ECU_LENGTHS[tag]
				combined[tag].fill(undefined)
			}
			if (!ECU_LENGTHS[tag] && pos > combined[tag].length) combined[tag].length = pos
			combined[tag][pos - 1] = value;
			continue
		}
		if (element.$name.startsWith('tires_')) {
			if (!value.includes(";")) continue
			// console.log("TPMS.DEBUG", {key, value})
			// let pre = value
			if (element.$name == 'tires_provisioning') value = TPMS.provisioning(value)
			else if (element.$name == 'tires_pressure') value = TPMS.pressure(value)
			else if (element.$name == 'tires_temperature') value = TPMS.temperature(value)
			else value = TPMS.get_default(value)
			if ($taip_tag.includes('.')) {
				// console.log({$taip_tag, pre, value})
				const [tag, pos] = $taip_tag.split(".");
				if (!complex[tag]) complex[tag] = {}
				value.split(",").map((tire) => {
					const [tpos, tvalue] = [tire.slice(0, 3), tire.slice(3)]
					if (!complex[tag][tpos]) {
						complex[tag][tpos] = []
						if (ECU_LENGTHS[tag] > complex[tag][tpos].length)
							complex[tag][tpos].length = ECU_LENGTHS[tag]
						complex[tag][tpos].fill(0)
					}
					if (!ECU_LENGTHS[tag] && pos > complex[tag][tpos].length) complex[tag][tpos].length = pos
					complex[tag][tpos][pos - 1] = tvalue;
				})
				continue
			}
		}
		txt += `;${$taip_tag}=${ecuFlag}${value}`;
	}
	for (const tag in combined) {
		let arr = combined[tag];
		let partial = partials.includes(tag)
		let value = `${arr.map(v => {
			if (v == undefined) return ""
			if (partial) return v
			else return `${ecuFlag}${v}`
		}).join(",")}`
		if (partial) value = `${ecuFlag}${value}`
		txt += `;${tag}=${value}`
		continue
	}
	for (const tag in complex) {
		let item = complex[tag]
		let sval = []
		for (const subtag in item) {
			let value = item[subtag]
			if (tag == 'ZY') {
				value[2] = "3"
				value[6] = "1"
			}
			sval.push(`${subtag}${value.join("")}`)
		}
		txt += `;${tag}=${ecuFlag}${sval.join(",")}`
	}
	let error_pgn = data['$ecu.feca_3-6']
	if (error_pgn) {
		let { spn, fmi, cm, oc } = {
			spn: data['$ecu.error_codes.spn'] || '0',
			fmi: data['$ecu.error_codes.fmi'] || '0',
			cm: data['$ecu.error_codes.cm'] || '0',
			oc: data['$ecu.error_codes.oc'] || '0',
		}
		let partial = `;ZC=${ecuFlag}1$${spn},${fmi},${cm},${oc}`
		if (!partial.endsWith('$0,0,0,0')) txt += partial
	}
	let mil_indicator = data['$ecu.feca_1.7']
	if (mil_indicator) {
		txt += `;YT=${ecuFlag}${mil_indicator},1,0`;
	}
	// console.log({ combined, complex, txt })
	return txt;
}
